@@ -1,12 +1,12 @@
+import typing
 from abc import ABC, abstractmethod
 from socket import AF_INET, SOCK_DGRAM, SOCK_STREAM, socket
 from typing import (
-    Any, BinaryIO, Generator, Generic, Iterable, List, Optional, TypeVar, cast
+    BinaryIO, Generator, Generic, Iterable, List, TypeVar, cast
 )
 
 from pyais.exceptions import InvalidNMEAMessageException
 from pyais.messages import NMEAMessage
-from pyais.util import FixedSizeDict
 
 F = TypeVar("F", BinaryIO, socket, None)
 DOLLAR_SIGN = ord("$")
@@ -45,28 +45,40 @@ class AssembleMessages(ABC):
         return next(iter(self))
 
     def _assemble_messages(self) -> Generator[NMEAMessage, None, None]:
-        queue: List[NMEAMessage] = []
+        buffer: typing.Dict[typing.Tuple[int, str], typing.List[typing.Optional[NMEAMessage]]] = {}
 
-        for line in self._iter_messages():
+        messages = self._iter_messages()
+        for line in messages:
 
-            # Be gentle and just skip invalid messages
             try:
                 msg: NMEAMessage = NMEAMessage(line)
             except InvalidNMEAMessageException:
+                # Be gentle and just skip invalid messages
                 continue
 
             if msg.is_single:
                 yield msg
-
-            # Assemble multiline messages
-            elif msg.is_multi:
-                queue.append(msg)
-
-                if msg.fragment_number == msg.message_fragments:
-                    yield msg.assemble_from_iterable(queue)
-                    queue.clear()
             else:
-                raise ValueError("Messages are out of order!")
+                # Instead of None use -1 as a seq_id
+                seq_id = msg.seq_id
+                if seq_id is None:
+                    seq_id = -1
+
+                # seq_id and channel make a unique stream
+                slot = (seq_id, msg.channel)
+
+                if slot not in buffer:
+                    # Create a new array in the buffer that has enough space for all fragments
+                    buffer[slot] = [None, ] * max(msg.fragment_count, 0xff)
+
+                buffer[slot][msg.frag_num - 1] = msg
+                msg_parts = buffer[slot][0:msg.fragment_count]
+
+                # Check if all fragments are found
+                not_none_parts = [m for m in msg_parts if m is not None]
+                if len(not_none_parts) == msg.fragment_count:
+                    yield NMEAMessage.assemble_from_iterable(not_none_parts)
+                    del buffer[slot]
 
     @abstractmethod
     def _iter_messages(self) -> Generator[bytes, None, None]:
@@ -170,93 +182,17 @@ class ByteStream(Stream[None]):
         yield from self.iterable
 
 
-class OutOfOrderByteStream(Stream[F], ABC):
-    """
-    Handles multipart NMEA that are delivered out of order.
-
-    This class is not attached to a datasource by default.
-    You need to subclass it and override _get_messages().
-    """
-
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        # create a fixed sized message queue
-        self._queue = FixedSizeDict(10000)
-        super().__init__(*args, **kwargs)
-
-    def __exit__(self, exc_type: object, exc_val: object, exc_tb: object) -> None:
-        del self._queue
-        super().__exit__(exc_type, exc_val, exc_tb)
-
-    def _split_nmea_header(self, msg: bytes) -> None:
-        """
-        Read the important parts of a NMEA header
-        """
-        parts: List[bytes] = msg.split(b',')
-        self.seq_id: int = int(parts[3]) if parts[3] else 0
-        self.fragment_offset: int = int(parts[2]) - 1
-        self.fragment_count: int = int(parts[1])
-
-    def _yield_complete(self) -> Optional[List[bytes]]:
-        """
-        Check if the message is complete and return it
-        """
-        # get all messages for the current sequence number
-        queue: List[bytes] = self._queue[self.seq_id][0:self.fragment_count]
-        if all(queue[0: self.fragment_count]):
-            # if all required messages are received yield them and free their space
-            del self._queue[self.seq_id]
-            return queue[0: self.fragment_count]
-        return None
-
-    def _add_to_queue(self, msg: bytes) -> None:
-        """
-        Append a new nmea message to queue
-        """
-        # MAX frag offset for any AIS NMEA is 9
-        msg_queue: List[Optional[bytes]] = ([None, ] * 9)
-        try:
-            # place the message at its correct position
-            msg_queue[self.fragment_offset] = msg
-        except IndexError:
-            # message is invalid clear it
-            del self._queue[self.seq_id]
-        self._queue[self.seq_id] = msg_queue
-
-    def _update_queue(self, msg: bytes) -> Optional[Iterable[bytes]]:
-        """
-        Update an existing message queue that is not complete yet.
-        Return a list of fully assembled messages if all required messages for a given sequence number are received.
-        """
-        msg_queue: List[bytes] = self._queue[self.seq_id]
-        msg_queue[self.fragment_offset] = msg
-        complete: Optional[List[bytes]] = self._yield_complete()
-        if complete:
-            yield from complete
-        return None
-
-    def _iter_messages(self) -> Generator[bytes, None, None]:
-        for msg in self.read():
-            # decode nmea header
-            self._split_nmea_header(msg)
-            if not self.seq_id:
-                yield msg
-            try:
-                complete = self._update_queue(msg)
-                if complete is not None:
-                    yield from complete
-
-            except KeyError:
-                # place item a correct pos and then store the list
-                self._add_to_queue(msg)
-
-
 class SocketStream(Stream[socket]):
     BUF_SIZE = 4096
 
+    def recv(self) -> bytes:
+        return b""
+
     def read(self) -> Generator[bytes, None, None]:
         partial: bytes = b''
         while True:
-            body = self._fobj.recv(self.BUF_SIZE)
+            body = self.recv()
+
             # Server closed connection
             if not body:
                 return None
@@ -272,20 +208,26 @@ class SocketStream(Stream[socket]):
             partial = lines[-1]
 
 
-class UDPStream(OutOfOrderByteStream[socket], SocketStream):
+class UDPReceiver(SocketStream):
 
     def __init__(self, host: str, port: int) -> None:
         sock: socket = socket(AF_INET, SOCK_DGRAM)
         sock.bind((host, port))
         super().__init__(sock)
 
+    def recv(self) -> bytes:
+        return self._fobj.recvfrom(self.BUF_SIZE)[0]
 
-class TCPStream(SocketStream):
+
+class TCPConnection(SocketStream):
     """
-     NMEA0183 stream via socket. Refer to
+     Read AIS data from a remote TCP server
      https://en.wikipedia.org/wiki/NMEA_0183
      """
 
+    def recv(self) -> bytes:
+        return self._fobj.recv(self.BUF_SIZE)
+
     def __init__(self, host: str, port: int = 80) -> None:
         sock: socket = socket(AF_INET, SOCK_STREAM)
         try:
